<?php

namespace App\Http\Controllers\Api;

use App\Models\Track;
use App\Repositories\TrackRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class NctController extends Controller
{
    protected $trackRepository;

    public function __construct(TrackRepository $trackRepository)
    {
        $this->trackRepository = $trackRepository;
        ini_set('max_execution_time', -1);
    }

    /**
     * API: http://localhost:8000/api/get-nct-top100
     * Deploy: https://vmusicchart.vn/crawler/api/get-nct-top100
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function getNctTop100(Request $request)
    {
        $url = config('api.nct.top100');
        $url_playlist_top100 = config('api.nct.playlist_top100');

        $tracks = $this->trackRepository->processNctGetTrackInfoFromTop100($url);
        $tracks_playlist = $this->trackRepository->processNctGetTrackInfoFromTop100Playlist($url_playlist_top100);
        $tracks_nct = Track::select(['id', 'id_nct', 'composer_list'])
            ->whereNotNull('id_nct')
            ->where('id_nct', '<>', '')
            ->get();
        $array_tracks_nct = [];
        foreach ($tracks_nct as $track) {
            $array_tracks_nct[$track['id_nct']] = $track['composer_list'];
        }

        $response = null;
        foreach ($tracks as $key => $track) {
            $link = null;
            $duration = 0;
            $check = $this->privateGeTrackLink($tracks_playlist, $track['id_nct']);
            if ($check) {
                $link = $check['link'];
                $duration = $check['duration'];
            }
            $response[] = $track;
            $response[$key]['link'] = $link;
            $response[$key]['duration'] = $duration;
            $response[$key]['str_duration'] = gmdate('i:s', $duration);
            if (array_key_exists($track['id_nct'], $array_tracks_nct)) {
                $response[$key]['composer'] = $array_tracks_nct[$track['id_nct']];
            } else {
                $response[$key]['composer'] = '?';
            }
        }

        return response()->json([
            'code' => 200,
            'message' => 'Lấy thông tin top 100 bài hát NCT thành công.',
            'data' => $response
        ]);
    }

    private function privateGeTrackLink($tracks_playlist, $track_id)
    {
        if (is_array($tracks_playlist) && count($tracks_playlist) > 0) {
            foreach ($tracks_playlist as $key => $val) {
                if ($val['id_nct'] == $track_id) {
                    return $val;
                }
            }
        }

        return null;
    }

}
